;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  directives  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		LIST P=16F84
		INCLUDE "p16f84.inc"
		ERRORLEVEL -224				;  supress annoying error message from tris
		__CONFIG _PWRTE_ON & _XT_OSC & _WDT_OFF	;  choose the configuration fuses
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  equates  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
MAX_POP  EQU d'3'            ;  The maximum number of pop the machine can contain
POP_COST EQU d'2'            ;  The cost of a pop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  data  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             CBLOCK h'0C'    ;  set up general registers starting at address 12
               tokens        ;  the current number of un-spent tokens
               pops          ;  the current number of available pop
               temp          ;  a general temp register
             ENDC

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  initialization  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             org h'0000'	
             ;...            ;  do general initialization
             clrf tokens     ;  start with no un-spent tokens
             movlw MAX_POP   ;  start with full pop
             movwf pops
             
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  main  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
main         call doToken
             call doPop
             call doRefill
             goto main

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  event routines  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
doToken      ;...            ;  test the sensor
             goto token_exit ;  if the event hasn't been initiated then quit

rule_2_1     movf pops, F    ;  don't accept tokens when pops is zero
             btfsc STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto token_no

rule_1_2     movlw POP_COST  ;  don't accept more than POP_COST tokens
             subwf tokens, W
             btfsc STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto token_no

token_yes    incf tokens     ;  update the system variables
             ;...            ;  do the work for the yes case
             goto token_exit
token_no     ;...            ;  do the work for the no case
token_exit   return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

doPop        ;...            ;  test the sensor
             goto pop_exit   ;  if the event hasn't been initiated then quit

rule_2_2     movf pops, F    ;  don't deliver pop when pops is zero
             btfsc STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto pop_no

rule_1_1     movlw POP_COST  ;  don't deliver pop unless tokens = POP_COST
             subwf tokens, W
             btfss STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto pop_no

pop_yes      clrf tokens     ;  update the system variables
             decf pops
             ;...            ;  do the work for the yes case
             goto pop_exit
pop_no       ;...            ;  do the work for the no case
pop_exit     return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

doRefill     ;...            ;  test the sensor
             goto refill_exit ; if the event hasn't been initiated then quit
             
             movlw MAX_POP   ;  update the system variables
             movwf pops
refill_exit  return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  end  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             end             ; end of program