;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  directives  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		LIST P=16F84
		INCLUDE "p16f84.inc"
		ERRORLEVEL -224				;  supress annoying error message from tris
		__CONFIG _PWRTE_ON & _XT_OSC & _WDT_OFF	;  choose the configuration fuses
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  equates  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
TOKEN_EVENT  EQU d'0'        ;  offset of token  in the sdata array
POP_EVENT    EQU d'1'        ;  offset of pop    in the sdata array
REFILL_EVENT EQU d'2'        ;  offset of refill in the sdata array
ENABLED      EQU d'0'        ;  zero indicates enablement
DISABLED     EQU h'f'        ;  fifteen indicates disablement

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  data  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             CBLOCK h'0C'    ;  set up general registers starting at address 12
               event         ;  an event that is about to occur in the plant
               decision      ;  the enablement decision of the last asked supervisor
               s1state       ;  the state of the supervisor1
               s2state       ;  the state of the supervisor2
               n1state       ;  the next state for supervisor1 if no one else disables the event
               n2state       ;  the next state for supervisor2 if no one else disables the event
               temp          ;  a general temp register
             ENDC

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  initialization  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             org h'0000'	
             ;...            ;  do general initialization
             clrf s1state    ;  initial supervisor state is zero
             clrf s2state    ;  initial supervisor state is zero
             
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  main  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
main         call doToken
             call doPop
             call doRefill
             goto main

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  event routines  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
doToken      ;...            ;  test the sensor
             goto token_exit ;  if the event hasn't been initiated then quit

             movlw TOKEN_EVENT 
             movwf event

             call supervisor1 ; if any supervisor disables then take the no branch
             btfsc decision, 0
             goto token_no

             call supervisor2 ; if any supervisor disables then take the no branch
             btfsc decision, 0
             goto token_no

token_yes    call super_commit
             ;...            ;  do the work for the yes case
             goto token_exit
token_no     ;...            ;  do the work for the no case
token_exit   return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

doPop        ;...            ;  test the sensor
             goto pop_exit   ;  if the event hasn't been initiated then quit

             movlw POP_EVENT 
             movwf event

             call supervisor1 ; if any supervisor disables then take the no branch
             btfsc decision, 0
             goto pop_no

             call supervisor2 ; if any supervisor disables then take the no branch
             btfsc decision, 0
             goto pop_no

pop_yes      call super_commit
             ;...            ;  do the work for the yes case
             goto pop_exit
pop_no       ;...            ;  do the work for the no case
pop_exit     return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

doRefill     ;...            ;  test the sensor
             goto refill_exit ; if the event hasn't been initiated then quit

             movlw REFILL_EVENT 
             movwf event

             call supervisor1 ; all supervisors must be notified
             call supervisor2
             call super_commit

refill_yes   ;...            ;  do the work
refill_exit  return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  supervisor1 subroutine  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  input:  assumes event = event currently being initiated
;;          i.e. TOKEN_EVENT, POP_EVENT or REFILL_EVENT
;;  output: sets decision = ENABLED or DISABLED
;;          also updates its state accordingly (in s1state)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
supervisor1  movf s1state, W ;  used determine which data to fetch from the s1data array
             movwf temp

             movlw s1data    ;  the start of the s1data array

             movf s1state, F ;  in state zero we want to skip the loop
             btfsc STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto $+4

             addlw d'3'      ;  adds to W the # to skip ahead to get the data for the current state
             decfsz temp, F
             goto $-2

             addwf event, W  ;  adds to W the # to skip ahead to get the data for the current event

             movwf FSR       ;  the address of the next_state data (INDF will now access the correct data)
             movlw DISABLED
             subwf INDF, W   ;  test if the transition is disabled
             btfsc STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto s1Disable

s1Enable     movf INDF, W    ;  update the next state
             movwf n1state
             movlw ENABLED   ;  mark as enabled
             movwf decision
             goto s1Exit

s1Disable    movlw DISABLED   ;  mark as disabled
             movwf decision

s1Exit       return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  supervisor2 subroutine  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  input:  assumes event = event currently being initiated
;;          i.e. TOKEN_EVENT, POP_EVENT or REFILL_EVENT
;;  output: sets decision = ENABLED or DISABLED
;;          also updates its state accordingly (in s2state)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
supervisor2  movf s2state, W ;  used determine which data to fetch from the s2data array
             movwf temp

             movlw s2data    ;  the start of the s2data array

             movf s2state, F ;  in state zero we want to skip the loop
             btfsc STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto $+4

             addlw d'3'      ;  adds to W the # to skip ahead to get the data for the current state
             decfsz temp, F
             goto $-2

             addwf event, W  ;  adds to W the # to skip ahead to get the data for the current event

             movwf FSR       ;  the address of the next_state data (INDF will now access the correct data)
             movlw DISABLED
             subwf INDF, W   ;  test if the transition is disabled
             btfsc STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto s2Disable

s2Enable     movf INDF, W    ;  update the next state
             movwf n2state
             movlw ENABLED   ;  mark as enabled
             movwf decision
             goto s2Exit

s2Disable    movlw DISABLED   ;  mark as disabled
             movwf decision

s2Exit       return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  commit all supervisors to the computed change of state  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
super_commit movf n1state, W
             movwf s1state
             movf n2state, W
             movwf s2state
             return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  supervisor data  (h'f' indicates disablement)  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

s1data       dw h'1', h'f', h'0'  ;  outgoing (token, pop, refill) from state 0
             dw h'2', h'f', h'1'  ;  outgoing (token, pop, refill) from state 1
             dw h'f', h'0', h'2'  ;  outgoing (token, pop, refill) from state 2

s2data       dw h'0', h'1', h'0'  ;  outgoing (token, pop, refill) from state 0
             dw h'1', h'2', h'0'  ;  outgoing (token, pop, refill) from state 1
             dw h'2', h'3', h'0'  ;  outgoing (token, pop, refill) from state 2
             dw h'f', h'f', h'0'  ;  outgoing (token, pop, refill) from state 3

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  end  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             end             ; end of program