;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  directives  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		LIST P=16F84
		INCLUDE "p16f84.inc"
		ERRORLEVEL -224				;  supress annoying error message from tris
		__CONFIG _PWRTE_ON & _XT_OSC & _WDT_OFF	;  choose the configuration fuses
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  equates  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
EVENT_TOKEN  EQU d'0'        ;  0th bit of the event register
EVENT_POP    EQU d'1'        ;  1st bit of the event register
EVENT_REFILL EQU d'2'        ;  2nd bit of the event register
ENABLED      EQU d'1'        ;  not zero indicates enablement
DISABLED     EQU d'0'        ;  zero indicates disablement

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  data  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             CBLOCK h'0C'    ;  set up general registers starting at address 12
               event         ;  an event that is about to occur in the plant
               decision      ;  the enablement decision of the last asked supervisor
               sstate1       ;  the state of the supervisor1
               sstate2       ;  the state of the supervisor2
               temp          ;  a general temp register
             ENDC

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  initialization  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             org h'0000'	
             ;...            ;  do general initialization
             clrf sstate1    ;  initial supervisor state is zero
             clrf sstate2    ;  initial supervisor state is zero
             
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  main  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
main         call doToken
             call doPop
             call doRefill
             goto main

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  event routines  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
doToken      ;...            ;  test the sensor
             goto token_exit ;  if the event hasn't been initiated then quit

             clrf event      ;  ask the supervisor
             bsf event, EVENT_TOKEN

             call supervisor1 ; if any supervisor disables then take the no branch
             btfss decision, 0
             goto token_no

             call supervisor2 ; if any supervisor disables then take the no branch
             btfss decision, 0
             goto token_no

token_yes    ;...            ;  do the work for the yes case
             goto token_exit
token_no     ;...            ;  do the work for the no case
token_exit   return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

doPop        ;...            ;  test the sensor
             goto pop_exit   ;  if the event hasn't been initiated then quit

             clrf event      ;  ask the supervisor
             bsf event, EVENT_POP

             call supervisor1 ; if any supervisor disables then take the no branch
             btfss decision, 0
             goto token_no

             call supervisor2 ; if any supervisor disables then take the no branch
             btfss decision, 0
             goto token_no

pop_yes      ;...            ;  do the work for the yes case
             goto pop_exit
pop_no       ;...            ;  do the work for the no case
pop_exit     return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

doRefill     ;...            ;  test the sensor
             goto refill_exit ; if the event hasn't been initiated then quit

             clrf event      ;  the supervisors need to be notified
             bsf event, EVENT_REFILL
             call supervisor1
             call supervisor2

refill_yes   ;...            ;  do the work
refill_exit  return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  supervisor1 subroutine  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  input: assume event = event currently being generated
;;         i.e. EVENT_TOKEN, EVENT_POP or EVENT_REFILL
;;  output: sets decision = ENABLED or DISABLED
;;          also updates its state accordingly (in sstate1)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
supervisor1  movlw ENABLED   ;  assume enablement
             movwf decision

             movf sstate1, W ;  determine which block of code to execute
             movwf temp
             movlw d'1'      ;  at least we'll want to jump ahead one line

             movf sstate1, F ;  in state zero we want to skip this loop
             btfsc STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto choose_evt1

             addlw d'9'      ;  sets W to the # to skip ahead to get to the current state
             decfsz temp, F
             goto $-2

choose_evt1  btfss event, EVENT_TOKEN
             goto $+3
             addlw d'0'
             goto jumpstate1
             btfsc event, EVENT_POP
             goto $+3
             addlw d'3'
             goto jumpstate1
             btfsc event, EVENT_REFILL
             goto $+3
             addlw d'6'
             goto jumpstate1
             
jumpstate1   addwf PCL, F    ;  goto the correct event block in the correct state

s1s0         movlw d'1'      ;  mark state change or disablement due to token event
             movwf sstate1             
             goto ssExit1
             
             movlw DISABLED  ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit1
             
             nop             ;  mark state change or disablement due to the refill event
             nop
             goto ssExit1

s1s1         movlw d'2'      ;  mark state change or disablement due to token event
             movwf sstate1             
             goto ssExit1
             
             movlw DISABLED  ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit1
             
             nop             ;  mark state change or disablement due to the refill event
             nop
             goto ssExit1

s1s2         movlw DISABLED  ;  mark state change or disablement due to token event
             movwf decision             
             goto ssExit1
             
             movlw d'0'      ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit1
             
             nop             ;  mark state change or disablement due to the refill event
             nop
             goto ssExit1

ssExit1      return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  supervisor2 subroutine  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  input: assume event = event currently being generated
;;         i.e. EVENT_TOKEN, EVENT_POP or EVENT_REFILL
;;  output: sets decision = ENABLED or DISABLED
;;          also updates its state accordingly (in sstate2)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
supervisor2  movlw ENABLED   ;  assume enablement
             movwf decision

             movf sstate2, W ;  determine which block of code to execute
             movwf temp
             movlw d'1'      ;  at least we'll want to jump ahead one line

             movf sstate2, F ;  in state zero we want to skip this loop
             btfsc STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto choose_evt2

             addlw d'9'      ;  sets W to the # to skip ahead to get to the current state
             decfsz temp, F
             goto $-2

choose_evt2  btfss event, EVENT_TOKEN
             goto $+3
             addlw d'0'
             goto jumpstate2
             btfsc event, EVENT_POP
             goto $+3
             addlw d'3'
             goto jumpstate2
             btfsc event, EVENT_REFILL
             goto $+3
             addlw d'6'
             goto jumpstate2
             
jumpstate2   addwf PCL, F    ;  goto the correct event block in the correct state

s2s0         nop             ;  mark state change or disablement due to token event
             nop             
             goto ssExit2
             
             movlw d'1'      ;  mark state change or disablement due to pop event
             movwf sstate2
             goto ssExit2
             
             nop             ;  mark state change or disablement due to the refill event
             nop
             goto ssExit2

s2s1         nop             ;  mark state change or disablement due to token event
             nop            
             goto ssExit2
             
             movlw d'2'      ;  mark state change or disablement due to pop event
             movwf sstate2
             goto ssExit2
             
             movlw d'0'      ;  mark state change or disablement due to the refill event
             movwf sstate2
             goto ssExit2

s2s2         nop             ;  mark state change or disablement due to token event
             nop            
             goto ssExit2
             
             movlw d'3'      ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit2
             
             movlw d'0'      ;  mark state change or disablement due to the refill event
             movwf sstate2
             goto ssExit2

s2s3         movlw DISABLED  ;  mark state change or disablement due to token event
             movwf decision
             goto ssExit2
             
             movlw DISABLED  ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit2
             
             movlw d'0'      ;  mark state change or disablement due to the refill event
             movwf sstate2
             goto ssExit2

ssExit2      return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  end  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             end             ; end of program