;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  directives  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		LIST P=16F84
		INCLUDE "p16f84.inc"
		ERRORLEVEL -224				;  supress annoying error message from tris
		__CONFIG _PWRTE_ON & _XT_OSC & _WDT_OFF	;  choose the configuration fuses
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  equates  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
TOKEN_EVENT  EQU d'0'        ;  offset of token  in the sdata array
POP_EVENT    EQU d'1'        ;  offset of pop    in the sdata array
REFILL_EVENT EQU d'2'        ;  offset of refill in the sdata array
ENABLED      EQU d'0'        ;  zero indicates enablement
DISABLED     EQU h'f'        ;  fifteen indicates disablement

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  data  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             CBLOCK h'0C'    ;  set up general registers starting at address 12
               event         ;  an event that is about to occur in the plant
               decision      ;  the enablement decision of the supervisor
               sstate        ;  the state of the supervisor
               temp          ;  a general temp register
             ENDC

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  initialization  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             org h'0000'	
             ;...            ;  do general initialization
             clrf sstate     ;  initial supervisor state is zero
             
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  main  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
main         call doToken
             call doPop
             call doRefill
             goto main

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  event routines  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
doToken      ;...            ;  test the sensor
             goto token_exit ;  if the event hasn't been initiated then quit

             movlw TOKEN_EVENT 
             movwf event
             call supervisor ;  ask the supervisor
             btfsc decision, 0
             goto token_no

token_yes    ;...            ;  do the work for the yes case
             goto token_exit
token_no     ;...            ;  do the work for the no case
token_exit   return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

doPop        ;...            ;  test the sensor
             goto pop_exit   ;  if the event hasn't been initiated then quit

             movlw POP_EVENT 
             movwf event
             call supervisor ;  ask the supervisor
             btfsc decision, 0
             goto pop_no

pop_yes      ;...            ;  do the work for the yes case
             goto pop_exit
pop_no       ;...            ;  do the work for the no case
pop_exit     return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

doRefill     ;...            ;  test the sensor
             goto refill_exit ; if the event hasn't been initiated then quit

             movlw REFILL_EVENT 
             movwf event
             call supervisor ;  notify the supervisor

refill_yes   ;...            ;  do the work
refill_exit  return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  supervisor subroutine  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  input:  assumes event = event currently being initiated
;;          i.e. TOKEN_EVENT, POP_EVENT or REFILL_EVENT
;;  output: sets decision = ENABLED or DISABLED
;;          also updates its state accordingly (in sstate)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
supervisor   movf sstate, W  ;  used determine which data to fetch from the sdata array
             movwf temp

             movlw sdata     ;  the start of the sdata array

             movf sstate, F  ;  in state zero we want to skip the loop
             btfsc STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto $+4

             addlw d'3'      ;  adds to W the # to skip ahead to get the data for the current state
             decfsz temp, F
             goto $-2

             addwf event, W  ;  adds to W the # to skip ahead to get the data for the current event

             movwf FSR       ;  the address of the next_state data (INDF will now access the correct data)
             movlw DISABLED
             subwf INDF, W   ;  test if the transition is disabled
             btfsc STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto disable

enable       movf INDF, W    ;  update the next state
             movwf sstate
             movlw ENABLED   ;  mark as enabled
             movwf decision
             goto ssExit

disable      movlw DISABLED   ;  mark as disabled
             movwf decision

ssExit       return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  supervisor data  (h'f' indicates disablement)  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

sdata        dw h'1', h'f', h'0'  ;  outgoing (token, pop, refill) from state 0
             dw h'2', h'f', h'1'  ;  outgoing (token, pop, refill) from state 1
             dw h'f', h'3', h'2'  ;  outgoing (token, pop, refill) from state 2
             dw h'4', h'f', h'0'  ;  outgoing (token, pop, refill) from state 3
             dw h'5', h'f', h'1'  ;  outgoing (token, pop, refill) from state 4
             dw h'f', h'6', h'2'  ;  outgoing (token, pop, refill) from state 5
             dw h'7', h'f', h'0'  ;  outgoing (token, pop, refill) from state 6
             dw h'8', h'f', h'1'  ;  outgoing (token, pop, refill) from state 7
             dw h'f', h'9', h'2'  ;  outgoing (token, pop, refill) from state 8
             dw h'f', h'f', h'0'  ;  outgoing (token, pop, refill) from state 9

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  end  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             end             ; end of program