;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  directives  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		LIST P=16F84
		INCLUDE "p16f84.inc"
		ERRORLEVEL -224				;  supress annoying error message from tris
		__CONFIG _PWRTE_ON & _XT_OSC & _WDT_OFF	;  choose the configuration fuses
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  equates  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
EVENT_TOKEN  EQU d'0'        ;  0th bit of the event register
EVENT_POP    EQU d'1'        ;  1st bit of the event register
EVENT_REFILL EQU d'2'        ;  2nd bit of the event register
ENABLED      EQU d'1'        ;  not zero indicates enablement
DISABLED     EQU d'0'        ;  zero indicates disablement

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  data  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             CBLOCK h'0C'    ;  set up general registers starting at address 12
               event         ;  an event that is about to occur in the plant
               decision      ;  the enablement decision of the supervisor
               sstate        ;  the state of the supervisor
               temp          ;  a general temp register
             ENDC

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  initialization  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             org h'0000'	
             ;...            ;  do general initialization
             clrf sstate     ;  initial supervisor state is zero
             
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  main  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
main         call doToken
             call doPop
             call doRefill
             goto main

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  event routines  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
doToken      ;...            ;  test the sensor
             goto token_exit ;  if the event hasn't been initiated then quit

             clrf event      ;  ask the supervisor
             bsf event, EVENT_TOKEN
             call supervisor
             btfsc decision, 0
             goto token_yes
             goto token_no
token_yes    ;...            ;  do the work for the yes case
             goto token_exit
token_no     ;...            ;  do the work for the no case
token_exit   return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

doPop        ;...            ;  test the sensor
             goto pop_exit   ;  if the event hasn't been initiated then quit

             clrf event      ;  ask the supervisor
             bsf event, EVENT_POP
             call supervisor
             btfsc decision, 0
             goto pop_yes
             goto pop_no
pop_yes      ;...            ;  do the work for the yes case
             goto pop_exit
pop_no       ;...            ;  do the work for the no case
pop_exit     return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

doRefill     ;...            ;  test the sensor
             goto refill_exit ; if the event hasn't been initiated then quit

             clrf event      ;  the supervisor needs to be notified
             bsf event, EVENT_REFILL
             call supervisor
refill_yes   ;...            ;  do the work
refill_exit  return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  supervisor subroutine  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  input: assume event = event currently being generated
;;         i.e. EVENT_TOKEN, EVENT_POP or EVENT_REFILL
;;  output: sets decision = ENABLED or DISABLED
;;          also updates its state accordingly (in sstate)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
supervisor   movlw ENABLED   ;  assume enablement
             movwf decision

             movf sstate, W  ;  determine which block of code to execute
             movwf temp
             movlw d'1'      ;  at least we'll want to jump ahead one line

             movf sstate, F  ;  in state zero we want to skip this loop
             btfsc STATUS, Z ;  zero bit is clear only when result is NOT zero
             goto choose_evt

             addlw d'9'      ;  sets W to the # to skip ahead to get to the current state
             decfsz temp, F
             goto $-2

choose_evt   btfss event, EVENT_TOKEN
             goto $+3
             addlw d'0'
             goto jumpstate
             btfsc event, EVENT_POP
             goto $+3
             addlw d'3'
             goto jumpstate
             btfsc event, EVENT_REFILL
             goto $+3
             addlw d'6'
             goto jumpstate
             
jumpstate    addwf PCL, F    ;  goto the correct event block in the correct state

ss0          movlw d'1'      ;  mark state change or disablement due to token event
             movwf sstate             
             goto ssExit
             
             movlw DISABLED  ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit
             
             nop             ;  mark state change or disablement due to the refill event
             nop
             goto ssExit

ss1          movlw d'2'      ;  mark state change or disablement due to token event
             movwf sstate             
             goto ssExit
             
             movlw DISABLED  ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit
             
             nop             ;  mark state change or disablement due to the refill event
             nop
             goto ssExit

ss2          movlw DISABLED  ;  mark state change or disablement due to token event
             movwf decision             
             goto ssExit
             
             movlw d'3'      ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit
             
             nop             ;  mark state change or disablement due to the refill event
             nop
             goto ssExit

ss3          movlw d'4'      ;  mark state change or disablement due to token event
             movwf sstate             
             goto ssExit
             
             movlw DISABLED  ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit
             
             movlw d'0'      ;  mark state change or disablement due to the refill event
             movwf sstate
             goto ssExit

ss4          movlw d'5'      ;  mark state change or disablement due to token event
             movwf sstate             
             goto ssExit
             
             movlw DISABLED  ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit
             
             movlw d'1'      ;  mark state change or disablement due to the refill event
             movwf sstate
             goto ssExit

ss5          movlw DISABLED  ;  mark state change or disablement due to token event
             movwf decision             
             goto ssExit
             
             movlw d'6'      ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit
             
             movlw d'2'      ;  mark state change or disablement due to the refill event
             movwf sstate
             goto ssExit

ss6          movlw d'7'      ;  mark state change or disablement due to token event
             movwf sstate             
             goto ssExit
             
             movlw DISABLED  ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit
             
             movlw d'0'      ;  mark state change or disablement due to the refill event
             movwf sstate
             goto ssExit

ss7          movlw d'8'      ;  mark state change or disablement due to token event
             movwf sstate             
             goto ssExit
             
             movlw DISABLED  ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit
             
             movlw d'1'      ;  mark state change or disablement due to the refill event
             movwf sstate
             goto ssExit

ss8          movlw DISABLED  ;  mark state change or disablement due to token event
             movwf decision             
             goto ssExit
             
             movlw d'9'      ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit
             
             movlw d'2'      ;  mark state change or disablement due to the refill event
             movwf sstate
             goto ssExit

ss9          movlw DISABLED  ;  mark state change or disablement due to token event
             movwf decision             
             goto ssExit
             
             movlw DISABLED  ;  mark state change or disablement due to pop event
             movwf decision
             goto ssExit
             
             movlw d'0'      ;  mark state change or disablement due to the refill event
             movwf sstate
             goto ssExit

ssExit       return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  end  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
             end             ; end of program